@@ -181,7 +181,7 @@ write_http_code_page (connection_t *con, int code, const char *msg)
 	char template_file[BUFSIZE];
 	char *filename;
 	
-	sprintf (template_file, "%d.html", code);
+	snprintf (template_file, BUFSIZE, "%d.html", code);
 	filename = get_template (template_file);
 
 	write_http_header (con->sock, code, msg);
@@ -289,7 +289,7 @@ url_decode (const char *intarget)
 
     len = ice_strlen (target);
     temp = (char *) nmalloc (len + 1);
-    strcpy (temp, target);
+    strncpy(temp, target, len+1);
 
     for (j = 0, run = 0; j < len; j++, run++) 
     {
@@ -327,7 +327,60 @@ url_decode (const char *intarget)
 }
 
 char * 
-url_encode (const char *str, char **result_p)
+html_escape (const char *str)
+{
+  const char *p;
+  char *q;
+  char *result;
+  int toescape= 0;
+
+  if (!str) {
+	  xa_debug (1, "WARNING: html_escape() called with NULL string");
+	  return NULL;
+  }
+
+  for (p = str; *p; p++) {
+    if ((unsigned char) (*p) == '&') toescape+=4;
+    if ((unsigned char) (*p) == '"') toescape+=5;
+    if ((unsigned char) (*p) == '<') toescape+=3;
+    if ((unsigned char) (*p) == '>') toescape+=3;
+  }
+  
+  result = (char  *) nmalloc (p - str + toescape + 1);
+
+  for (q = result, p = str; *p; p++) {
+      unsigned char a = *p;
+      if (a == '&') {
+      	  *q++ = '&';
+      	  *q++ = 'a';
+      	  *q++ = 'm';
+      	  *q++ = 'p';
+      	  *q++ = ';';
+      } else if (a == '"') {
+      	  *q++ = '&';
+      	  *q++ = 'q';
+      	  *q++ = 'u';
+      	  *q++ = 'o';
+      	  *q++ = 't';
+      	  *q++ = ';';
+      } else if (a == '<') {
+      	  *q++ = '&';
+      	  *q++ = 'l';
+      	  *q++ = 't';
+      	  *q++ = ';';
+      } else if (a == '>') {
+      	  *q++ = '&';
+      	  *q++ = 'g';
+      	  *q++ = 't';
+      	  *q++ = ';';
+      } else *q++ = *p;
+    }
+  *q++ = 0; 
+  return result;
+}
+
+char * 
+url_encode (const char *str, char** result_p)
 {
   const char *p;
   char *q;
@@ -571,6 +624,23 @@ http_admin_command (connection_t *con, request_t *req)
 		free_variables (request_vars);
 		return 1;
 	} else {
+		/* If a relay source is updating its metadata it will be
+		 * using the crypted encoder_password */
+		const char* inmount;
+		connection_t* source;
+
+		if (inpass &&
+		    (inmount = get_variable (request_vars, "mount")) &&
+		    (source = get_source_with_mount (inmount)) &&
+		    (source->type != encoder_e) && 
+		    (ice_strcmp (inmode, "updinfo") == 0) &&
+		    (ice_strcmp (info.encoder_pass, inpass) == 0))
+		{
+		  handle_http_admin_command (con, request_vars);
+		  free_variables (request_vars);
+		  return 1;
+		}
+		
 		free_variables (request_vars);
 		return 0;
 	}
@@ -598,16 +668,16 @@ display_admin_page (connection_t *con, request_t *req)
 
 	if (commandptr)
 	{
-		strcpy (commandstr, commandptr);
+		strncpy(commandstr, commandptr, BUFSIZE);
 		comreq.arg = commandstr;
 		do
 		{
-			sprintf (argstring, "argument%d", argcount);
+			snprintf(argstring, BUFSIZE, "argument%d", argcount);
 			commandptr = get_variable (request_vars, argstring);
 			if (commandptr)
 			{
-				sprintf (buff, "%s %s", comreq.arg, commandptr);
-				strcpy (comreq.arg, buff);
+				snprintf(buff, BUFSIZE, "%s %s", comreq.arg, commandptr);
+				strncpy(comreq.arg, buff, BUFSIZE);
 			}
 			argcount++;
 		} while (argcount < 10);
@@ -638,9 +708,9 @@ display_admin_page (connection_t *con, request_t *req)
 	zero_request (&checkreq);
 
 	if (com && com->oper)
-		strcpy (checkreq.path, "/oper");
+		strncpy(checkreq.path, "/oper", BUFSIZE);
 	else
-		strcpy (checkreq.path, "/admin");
+		strncpy(checkreq.path, "/admin", BUFSIZE);
 
 	if (info.allow_http_admin == 0 || need_authentication (&checkreq))
 	{
@@ -951,7 +1021,7 @@ http_odd (char *command, connection_t *clicon, connection_t *sourcecon, const ch
 		return NULL;
 	}
 
-	sprintf (endat, "@ENDFOR %s@", command);
+	snprintf(endat, BUFSIZE, "@ENDFOR %s@", command);
 
 	nextline = strchr (ptr, '\n');
 
@@ -997,7 +1067,7 @@ http_even (char *command, connection_t *clicon, connection_t *sourcecon, const c
 		return NULL;
 	}
 	
-	sprintf (endat, "@ENDFOR %s@", command);
+	snprintf(endat, BUFSIZE, "@ENDFOR %s@", command);
 
 	nextline = strchr (ptr, '\n');
 	
@@ -1155,7 +1225,7 @@ http_loop_sources (char *ident, connection_t *clicon, const char *ptr, int fd, v
 	      sock_write (clicon->sock, "No sources available<br>");
       else
 	      fd_write (fd, "No sources available<br>");
-      sprintf (buf, "@ENDFOR %s@", ident);
+      snprintf(buf, BUFSIZE, "@ENDFOR %s@", ident);
       return skip_after (ptr, buf);
     }
   
@@ -1209,7 +1279,7 @@ http_loop_admins (char *ident, connection_t *clicon, const char *ptr, int fd, va
 	      sock_write (clicon->sock, "No admins available<br>");
       else
 	      fd_write (fd, "No admins available<br>");
-      sprintf (buf, "@ENDFOR %s@", ident);
+      snprintf(buf, BUFSIZE, "@ENDFOR %s@", ident);
       return skip_after (ptr, buf);
     }
   
@@ -1252,7 +1322,7 @@ http_loop_directory (char *ident, connection_t *clicon, const char *ptr, int fd,
   { 
 	  xa_debug (2, "DEBUG: http_loop_directory(): Could not open directory");
 	  sock_write (clicon->sock, "No such directory<br>");
-	  sprintf (buf, "@ENDFOR %s@", ident);
+	  snprintf(buf, BUFSIZE, "@ENDFOR %s@", ident);
 	  return skip_after (ptr, buf);	  
   }
 
@@ -1260,7 +1330,7 @@ http_loop_directory (char *ident, connection_t *clicon, const char *ptr, int fd,
   
   while (dir_findnext (od))
   {
-	  sprintf (fullfilename, "%s/%s", directoryname, od->filename);
+	  snprintf(fullfilename, BUFSIZE, "%s/%s", directoryname, od->filename);
 	  if ((stat (fullfilename, &st) == 0) && wild_match ((unsigned char *)"*.mp3", (unsigned char *)od->filename))
 	  {
 		  char *escaped;
@@ -1299,7 +1369,7 @@ http_loop_listeners (char *ident, connection_t *clicon, const char *ptr, int fd,
 	      sock_write (clicon->sock, "No listeners available<br>");
       else
 	      fd_write (fd, "No listeners available<br>");
-      sprintf (buf, "@ENDFOR %s@", ident);
+      snprintf(buf, BUFSIZE, "@ENDFOR %s@", ident);
       return skip_after (ptr, buf);
     }
   
@@ -1319,7 +1389,7 @@ http_loop_listeners (char *ident, connection_t *clicon, const char *ptr, int fd,
 	  add_varpair2 (variables, nstrdup (ident), ice_itoa (i));
 	  add_varpair2 (variables, ice_cat (ident, ".id"), ice_itoa (travclients->id));
 	  add_varpair2 (variables, ice_cat (ident, ".host"), nstrdup (con_host (travclients)));
-	  add_varpair2 (variables, ice_cat (ident, ".user_agent"), nstrdup (get_user_agent (travclients)));
+	  add_varpair2 (variables, ice_cat (ident, ".user_agent"), nstrdup (html_escape(get_user_agent (travclients))));
 	  add_varpair2 (variables, ice_cat (ident, ".writebytes"), ice_utoa (travclients->food.client->write_bytes));
 	  add_varpair2 (variables, ice_cat (ident, ".connecttime"), nstrdup (nice_time (get_time() - travclients->connect_time, buf)));
 	  endptr = parse_template_file (clicon, NULL, runptr, fd, variables);
@@ -1435,7 +1505,7 @@ http_display (com_request_t *req)
 		return 0;
 	}
 
-	sprintf (command, "%s.html", arg);
+	snprintf(command, BUFSIZE+4, "%s.html", arg);
 
 	file = get_template (command);
 
@@ -1482,7 +1552,7 @@ http_change (com_request_t *req)
 		return 0;
 	}
 
-	sprintf (helpfile, "%s.html", arg);
+	snprintf(helpfile, BUFSIZE, "%s.html", arg);
 
 	file = get_template (helpfile);
 	
